﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using Celeste;
using Monocle;
using Microsoft.Xna.Framework;
using Celeste.Mod;
using System.Collections;
using System.Reflection;
using Celeste.Mod.Entities;

namespace Celeste.Mod.FarewellAdvent.Entities
{
	[Tracked]
	public class ConquerorDash : Entity
	{
		private Sprite Sprite, lightning;
		private bool lightningVisible;
		private Level level;
		private float attackSpeed;
		public ConquerorDash()
		{
			Add(Sprite = FarewellAdventModule.SpriteBank.Create("conqueror_boss"));
			Add(lightning = FarewellAdventModule.SpriteBank.Create("boss_lightning_charge"));
			lightning.Visible = false;
			lightning.OnFinish = delegate
			{
				lightningVisible = false;
			};
			Collider = new Hitbox(30f, 24f, -15f, -12f);
			Add(new PlayerCollider(OnPlayer, Collider));
		}

		public override void Awake(Scene scene)
		{
			base.Awake(scene);
			level = SceneAs<Level>();
		}

		public override void Added(Scene scene)
		{
			base.Added(scene);
			Add(new Coroutine(DashRoutine()));
		}

		private void OnPlayer(Player player)
		{
			player.Die((player.Center - base.Center).SafeNormalize());
		}

		private void Appear()
		{
			Sprite.Play("appear");
			Sprite.Scale = new Vector2(1f, 1f);
			Visible = true;
		}

		private IEnumerator DashRoutine()
		{
			X = level.Camera.Left + 20f;
			Player player = level.Tracker.GetEntity<Player>();
			if (player != null)
			{
				CenterY = player.CenterY;
			}
			Appear();
			yield return 0.1f;
			Sprite.Play("xcharge");
			Audio.Play("event:/ricky06/FarewellAdvent/cb-horizontal-charge");
			lightningVisible = true;
			lightning.Play("once", restart: true);
			Sprite.Scale.X *= -1;
			yield return 0.29f;

			attackSpeed = 0f;
			Sprite.Play("xattack");
			Audio.Play("event:/ricky06/FarewellAdvent/cb-horizontal-dash-fast");

			while (true)
			{
				X += attackSpeed * Engine.DeltaTime;
				attackSpeed = Calc.Approach(attackSpeed, 1000f, 4000f * Engine.DeltaTime);
				if (X >= level.Camera.Right + 48f)
				{
					Visible = false;
					break;
				}

				if (base.Scene.OnInterval(0.05f))
				{
					TrailManager.Add(this, Color.Purple * 0.8f, 1f, frozenUpdate: false, useRawDeltaTime: false);
				}
				yield return null;
			}

		}

		public override void Render()
		{
			base.Render();
			if (lightningVisible)
			{
				lightning.RenderPosition = new Vector2(level.Camera.Left, base.Top + 16f);
				lightning.Render();
			}
		}

	}
}
